/* HiDevice */

#import <Cocoa/Cocoa.h>
#import "MTCoreAudio.h"

#define StandardSampleRate		44100.0
#define StandardBufferSize		512

#define RecordSourceMicrophone	@"Internal microphone"
#define RecordSourceDigital		@"Digital In"
#define RecordSourceLinein		@"Line In"						//下の２つはこれに変換される
#define RecordSourceMicOrLine	@"External microphone/Line In"
#define RecordSourceSoundIn		@"Sound Input"

#define PlaySourceHeadphones	@"Headphones"
#define PlaySourceSpeakers		@"Internal speakers"

#define DEF_RECORDSOURCE		0
#define DEF_PLAYSOURCE			0
#define DEF_MONITORSOURCE		0
#define DEF_USEHIQUALITY		NO

extern NSString *KEY_RECORDSOURCE;
extern NSString *KEY_PLAYSOURCE;
extern NSString *KEY_MONITORSOURCE;
extern NSString *KEY_USEHIQUALITY;

@class HiControler;
@class MTConversionBuffer;

@interface HiDevice : NSObject

{
	HiControler			*controler;			//
	
	MTCoreAudioDevice	*recordDevice;		//録音用入力デバイス
	int					recordSource;		//入力装置番号
	MTAudioBuffer		*recordBuffer;		//録音用バッファ
	AudioBufferList		*convertBuffer;		//入力変換用バッファ
	double				recordSampleRate;		//サンプリングレート
	int					recordBufferSize;		//入出力用バッファのフレーム数
	int					recordChannels;
	BOOL				useHighQuality;		//入力装置最大のサンプルレートを使用する
	
	MTCoreAudioDevice	*playDevice;		//再生出力用デバイス
	int					playSource;		//出力装置番号
	double				playSampleRate;		//サンプリングレート
	int					playBufferSize;		//入出力用バッファのフレーム数
	int					playChannels;
	
	MTCoreAudioDevice	*monitorDevice;		//モニター用出力デバイス
	int					monitorSource;		//モニター用入力デバイス
	double				monitorSampleRate;
	int					monitorBufferSize;
	MTAudioBuffer		*monitorBuffer;		//モニター用バッファ
	AudioBufferList		*monitorConvertBuffer;
	int					monitorChannels;
	MTConversionBuffer	*monitorConverter;
	
	MTCoreAudioDevice	*levelDevice;	//レベルメータ用入力デバイス
	
	//起動状態
	BOOL	awakeError;
}

//********************
// 初期化
//********************

- (void)initRecordDevice;					//オーディオデバイス関係初期設定
- (void)initPlayDevice;
- (void)initMonitorDevice;
- (void)saveUserDefaults;

//********************
// ユーティリティ
//********************

- (void)recordDeviceStart;
- (double)recordSampleRateWithSource:(int)source;
- (double)recordSampleRateWithSource:(int)source quality:(int)number;
- (double)sampleRateWithNominal:(double)nominal quality:(int)number;
- (MTCoreAudioDevice *)recordDeviceWithSource:(int)source;
- (int)channelsOfPlayDevice:(MTCoreAudioDevice *)device;	//入力装置のチャネル数を返す
- (int)channelsOfRecordDevice:(MTCoreAudioDevice *)device;	//出力装置のチャネル数を返す

//=============
// コンバータ関係
//=============

- (void)newMonitorConverter;

//=========
// 録音関係
//==========

- (void)renewRecordSource;	//入力装置を設定（または変更）する
- (int)checkRecordSource:(int)inSource;
- (void)noRecordDeviceAlert;						//使用できる入力装置がない場合の処理
- (int)recordSourceCount;							//入力装置の数（入力ソースが複数あるものは入力ソースの数）
- (int)deviceNumberOfRecordSource:(int)inRecordSource;	//ソース番号から入力装置の番号を得る
- (NSString *)sourceNameOfRecordSource:(int)inRecordSource;	//ソース番号からソース名または装置名を得る
- (NSString *)japanSourceNameOfRecordSource:(int)sourceNumber;
- (NSString *)presentJapanRecordSourceName;		//現在の日本語入力ソース名を得る(入力ソース名がない場合は入力装置名を返す)
- (NSArray *)usableJapanRecordSourceNames;	//利用可能な入力ソースの一覧（日本語）、ポップアップメニューの作成時に使われる
- (NSArray *)usableRecordSourceNames;//指定の入力装置の使用可能な入力ソース名一覧を得る
- (NSString *)presentRecordSourceName;			//現在の入力ソース名を得る（入力ソース名がなければ入力装置名を返す）
- (BOOL)isPresentInternalRecord;					//現在の入力ソースは内蔵マイク？
- (BOOL)isPresentExternalRecord;					//現在の入力ソースはライン入力？
- (BOOL)isPresentDigitalRecord;					//現在の入力ソースはデジタル入力？
- (NSString *)convertJapanRecordSourceName:(NSString *)sourceName;	//ソース名を日本語に変換する
- (NSArray *)representRecordName:(NSArray *)inNames;	//同じ入力名があれば代表のみ返す
- (BOOL)isRecordableDevice:(MTCoreAudioDevice *)device;	//指定のデバイスは録音に使えるかどうかを知る
- (double)sampleRateOfDevice:(MTCoreAudioDevice *)device;	//入力装置のサンプルレートを返す
- (int)bufferSizeOfDevice:(MTCoreAudioDevice *)device;	//入力装置のバッファサイズ（フレーム数）を返す

//=========
// 再生関係
//==========

- (void)renewPlaySource;	//出力装置を設定（または変更）する
- (int)checkPlaySource:(int)inSource;
- (int)playSourceCount;							//出力装置の数（入力ソースが複数あるものは入力ソースの数）
- (void)noPlayDeviceAlert;						//使用できる入力装置がない場合の処理
- (int)deviceNumberOfPlaySource:(int)inPlaySource;	//ソース番号から出力装置の番号を得る
- (NSString *)sourceNameOfPlaySource:(int)inPlaySource;	//ソース番号から出力装置のソース名または装置名を得る
- (NSArray *)usableJapanPlaySourceNames;	//利用可能な出力ソースの一覧（日本語）、ポップアップメニューの作成時に使われる
- (NSArray *)usablePlaySourceNames;//指定の出力装置の使用可能な出力ソース名一覧を得る
- (NSString *)convertJapanPlaySourceName:(NSString *)sourceName;	//ソース名を日本語に変換する
- (NSArray *)representPlayName:(NSArray *)inNames;	//同じ出力名があれば代表のみ返す
- (BOOL)isPresentInternalPlay;
- (void)playDevicePause:(BOOL)inBool;
- (BOOL)isPlayableDevice:(MTCoreAudioDevice *)device;	//指定のデバイスは音声出力に使えるかどうかを知る

//===========
// モニター関係
//===========

- (void)renewMonitorSource;	//モニター装置を設定（または変更）する
- (BOOL)isPresentInternalMonitor;	//現在のモニター出力は内蔵？
- (void)monitorDevicePaused:(BOOL)inBool;

//********************
// アクセッサ
//********************

- (int)recordSource;
- (void)setRecordSource:(int)inSource;
- (void)changeRecordSource:(int)inSource;
- (int)playSource;
- (void)setPlaySource:(int)inSource;
- (void)changePlaySource:(int)inSource;
- (int)monitorSource;
- (void)setMonitorSource:(int)inSource;
- (void)changeMonitorSource:(int)inSource;
- (BOOL)awakeError;
- (MTCoreAudioDevice *)recordDevice;
- (MTCoreAudioDevice *)levelDevice;
- (MTCoreAudioDevice *)playDevice;
- (MTCoreAudioDevice *)monitorDevice;
- (MTAudioBuffer *)monitorBuffer;
- (MTAudioBuffer *)recordBuffer;
- (AudioBufferList *)convertBuffer;
- (AudioBufferList *)monitorConvertBuffer;
- (double)recordSampleRate;
- (int)recordBufferSize;		//
- (int)recordChannels;
- (double)playSampleRate;
- (int)playBufferSize;
- (BOOL)useHighQuality;
- (void)setUseHighQuality:(BOOL)inBool;
- (double)monitorSampleRate;
- (int)monitorBufferSize;
- (MTConversionBuffer *)monitorConverter;

//********************
// アクション
//********************

@end
