/* HiDutyLog */

#import <Cocoa/Cocoa.h>

#define LOGBOOK_FILENAME	@"HiSoundDutyLog.txt"
#define LOGHEADER_LENGTH	15
#define PATROL_INTERVAL		300.0

#define HiLogEventNotification			@"HiLogEventNotification"
#define LogEvent_DutyStart				@"DutyStart"
#define LogEvent_DutyFinish				@"DutyFinish"
#define LogEvent_RecordingStart			@"RecordingStart"
#define LogEvent_RecordingStop			@"RecordingStop"
#define LogEvent_RecordingSleep			@"RecordingSleep"
#define LogEvent_HourSwitchChange		@"HourSwitchChange"
#define LogEvent_SampleRateChange		@"SampleRateChange"
#define LogEvent_ChannelsChange			@"ChannelsChange"
#define LogEvent_InputDeviceChange		@"InputDeviceChange"
#define LogEvent_InputSourceChange		@"InputSourceChange"
#define LogEvent_OutputSourceChange		@"InputSourceChange"
#define LogEvent_RecordModeChange		@"RecordModeChange"
#define LogEvent_PreTimeChange			@"PreTimeChange"
#define LogEvent_DelayTimeChange		@"DelayTimeChange"
#define LogEvent_MarkerSwitchChange		@"MarkerSwitchChange"
#define LogEvent_MarkerPeriodChange		@"MarkerPeriodChange"
#define LogEvent_MarkerSoundChange		@"MarkerSoundChange"
#define LogEvent_RecordFolderChange		@"RecordFolderChange"
#define LogEvent_MaxRecordChange		@"MaxRecordChange"
#define LogEvent_KeepPeriodChange		@"KeepPeriodChange"
#define LogEvent_SoundFileCopy			@"SoundFileCopy"
#define LogEvent_SoundFileDelete		@"SoundFileDelete"
#define LogEvent_SomeErrorHappen		@"SomeErrorHappen"
#define LogEvent_SomeReportHappen		@"SomeReportHappen"
#define LogEvent_Sleepin				@"SleepIn"
#define LogEvent_Wakeup					@"WakeUp"

#define DEF_WRITELOGBOOK	YES
#define DEF_TROUBLETERMINATION	NO

extern NSString *KEY_WRITELOGBOOK;
extern NSString *KEY_TROUBLETERMINATION;
extern NSString *KEY_LASTPATROLDATE;
extern NSString *KEY_ERRORMESSAGE;

@class HiControler;

@interface HiDutyLog : NSObject
{
	HiControler *controler;
	
	NSFileHandle	*logBook;
	
	BOOL			writeLog;		//ログを付けるかどうかのフラグ
	NSTimer			*patrolTimer;
	
	//記録項目全体
	NSCalendarDate	*dutyStart;		//記録開始日時
	int				serialNum;		//一連番号
	
	//記録項目細部
	NSCalendarDate	*startDate;		//録音開始時刻
}

- (void)saveUserDefaults;
- (void)initLogItem;
- (void)patrolTimerPeriodic:(NSTimer *)timer;
- (void)logEventNotificationHandler:(NSNotification *)note;
- (NSString *)recordModeString:(int)inMode;
- (int)markerPeriodTime:(int)inPeriod;
- (NSString *)channelsString:(int)channels;
- (NSString *)presentSourceString;
- (NSString *)methodString:(int)inMethod;
- (NSString *)dateString:(NSCalendarDate *)inDate;
- (NSString *)periodStringFrom:(NSCalendarDate *)fromDate to:(NSCalendarDate *)toDate;
- (NSString *)periodStringFromSeconds:(double)inSeconds;
- (BOOL)isLogFile:(NSString *)fileName;

- (BOOL)writeLog;
- (void)setWriteLog:(BOOL)inBool;

@end
