/* HiKeeper */

#import <Cocoa/Cocoa.h>

#define kTestFileExtension	@"test"
#define kCompressQueMax		10
#define Def_TestFilePath	@"/System/Library/Sounds"
#define Def_TestFileName	@"Blow.aiff"
#define Key_TestFilePath	@"SampleDataFilePath"
#define Key_TestFileName	@"SampleDataFileName"

#define Def_KeepOperation	NO
#define Def_KeepFolder		@"~/Documents/HiSoundDutyPreserve"
#define Def_KeepPeriod		10

#define Key_CompressError	@"CompressErrorMessage"
#define CompressingError	@"CompresserExecuteError"
#define NoCompresserError	@"CompresserNoExistError"

#define Key_KeepOperation	@"KeepOperation"
#define Key_KeepFolder		@"KeepFileKeepFolder"
#define Key_KeepPeriod		@"KeepFileKeepPeriod"
//#define Def_CommandPath		@"/usr/bin/gzip"
#define Def_CommandPath		@"/usr/bin/bzip2"
#define	Key_CommandPath		@"CommandPathString"
#define	Def_ArgumentStr		@"-f %@"
#define	Key_ArgumentStr		@"CommandArgumentString"
#define Def_MonitorCommand	YES
#define	Key_MonitorCommand	@"MonitorCommandLine"
#define Def_CompressRatio	0.66
#define Key_CompressRatio	@"CompressRatio"
#define kExpandMenuItem		20

#define Def_ExpandFolder		@"~/DeskTop"
#define Key_ExpandFolder		@"KeepFileExpandFolder"

#define Note_CompressComplete	@"CompressCompleteNotification"
#define Note_CompressRatio		@"CompressRatioNotification"
#define Note_DataKeepComplete	@"DataKeepCompleteNotification"

@class HiRecorder;
@class HiKeepList;
@class HiMechanic;

@interface HiKeeper : NSObject
{
	//HiKeeperで必要なHiRecorderのメソッドは
	//needPerSecond, maxRecordTime, headerString の3つ
	HiRecorder	*recorder;
	HiMechanic	*mechanic;
	NSProgressIndicator	*compressIndicator;
	
	BOOL		keepOperation;	//圧縮保存するかどうかのスイッチ
	NSString	*keepFolder;	//保存場所
	int			keepPeriod;		//保存期間（単位は日）
	NSString	*commandPath;	//圧縮コマンド
	NSString	*argumentStr;	//圧縮コマンド細部指定文字列
	BOOL		monitorCommand;	//圧縮コマンドのモニタースイッチ
	float		compressRatio;	//圧縮率
	NSMutableArray *commandQue;	//圧縮コマンドのキュー
	BOOL		compressing;	//圧縮コマンド実行中のフラグ

	HiKeepList	*dataSource;
	NSTableView *dataView;
	NSTextField	*totalTimeText;
	NSTextField	*totalSizeText;
	NSTextField *totalFilesText;
	NSString	*expandFolder;
	NSTabView	*tabView;
}

//********************
// 初期化
//********************

+ (void)initialize;
- (void)awakeFromNib;

//********************
// ユーティリティ
//********************

- (double)totalDataSize;
- (void)compressDataFile:(NSString *)filePath;
- (void)compressFileThread:(NSString *)filePath;
- (void)compressThreaCompleteHandle:(NSNotification *)note;
- (void)completeCheckKeepActionHandle:(NSNotification *)note;
- (OSStatus)execTaskWithPath:(NSString *)inCommand
				   arguments:(NSArray *)inArguments;
- (BOOL)secureKeepSpace;	//必要な保存ファイル領域を確保する
- (double)totalKeepTime;	//現在保存されているファイルの総時間
- (double)totalKeepSpace;	//現在保存されているファイルの総容量
- (BOOL)deleteOlderKeepFile;	//録音日時の古いファイルを１つ削除する
- (double)needFreeKeepSpace;	//現在の設定で1つの録音データを保存するのに必要な領域
- (BOOL)isKeepDataFile:(NSString *)fileName;	//HiSoundDutyのファイルかどうかをチェックする
- (double)freeSystemSpace;	//システムの使用可能な空き領域
- (int)headerLength;
- (BOOL)isCompressing;
- (double)totalSecondOfFile:(NSString *)fileName;	

//********************
// アクセッサ
//********************

- (BOOL)keepOperation;
- (void)setKeepOperation:(BOOL)inOperation;
- (NSString *)keepFolder;
- (void)setKeepFolder:(NSString *)inFolder;
- (int)keepPeriod;
- (void)setKeepPeriod:(int)inPeriod;
- (NSString *)commandPath;
- (void)setCommandPath:(NSString *)inCommand;
- (NSString *)argumentStr;
- (void)setArgumentStr:(NSString *)inArgument;
- (BOOL)monitorCommand;
- (void)setMonitorCommand:(BOOL)inBool;
- (float)compressRatio;
- (void)setCompressRatio:(float)inRatio;

- (void)mainTabSelected;
- (void)mainTabUnselected;
- (void)renewListAndControl;
- (void)initDataSource;
- (void)changeSelectRow;
- (void)actPlayback:(id)sender;
- (void)actCopy:(id)sender;
- (void)actDelete:(id)sender;

//********************
// デリゲートメソッド
//********************

//- (BOOL)validateMenuItem:(NSMenuItem*)item;	//メニューのイネーブル設定

@end
