/* HiMechanic */

#import <Cocoa/Cocoa.h>
#import <AudioToolbox/AudioToolbox.h>

enum {
	Form_AIFF	= 0,
	Form_WAVE	= 1
};

#define EXTENSION_AIFF	@"aif"
#define EXTENSION_TIFF	@"tif"
#define EXTENSION_SOUND	@"aiff"

#define TEMP_FILENAME	@".HiSoundDutyTempFile"
#define RELAY_FILENAME	@".HiSoundDutyRelayFile"
#define HEADER_STRING		@"_HiSoundDuty_"
#define HIDEHEADER_STRING	@".HiSoundDuty_"
#define HEADER_LENGTH		13
#define BODY_LENGTH			22
#define EXTENSION_LENGTH	4
#define OFFSET_HEADER			256
#define OFFSET_RECORDDATE		OFFSET_HEADER+HEADER_LENGTH	//20050524_000000~000000
#define OFFSET_NOTESTR			OFFSET_RECORDDATE+22		//備考
#define MAXLENGTH_NOTESTR		128

#define DEF_DIRPATH		@"~/Documents/HiSoundDutyRecord"
#define DEF_SOUNDFORM	Form_AIFF
#define DEF_MAXRECORDTIME	(1*60*60)	//１つのファイルの録音時間
#define DEF_MAXTOTALTIME	(24*60*60)	//保存するファイルの時間

extern NSString	*KEY_DIRPATH;
extern NSString *KEY_SOUNDFORM;
extern NSString *KEY_MAXRECORDTIME;
extern NSString *KEY_MAXTOTALTIME;
#define Def_AdviceOperation	NO
#define Key_AdviceOperation	@"BackupAdviceOperation"
#define Def_AdviceDataSize	4700
#define Key_AdviceDataSize	@"BackupAdviceDataSize"
#define kBackupFolderName	@"HiSoundDutyBackup"

@class HiControler;
@class HiKeeper;

@interface HiMechanic : NSObject
{
	//ファイル関係
	//IBOutlet
	HiControler	*controler;
	HiKeeper	*keeper;
	
	//ファイル出力用（録音）
	NSString	*recordFolder;		//音声ファイルの保存場所
	int			soundForm;		//音声ファイルの記録形式
	double		maxRecordTime;	//最大録音時間（単位は秒)
	double		maxTotalTime;	//最大累計録音時間
    AudioStreamBasicDescription audioFormatOutput;	//
	
	//ファイル入力用（再生）
	AudioStreamBasicDescription audioFormatInput;
	SInt64 numDatas;	//データの大きさ

	//バックアップ勧告用
	NSPanel		*adviceAlertPanel;
	NSTextField	*adviceSizeText;
	NSTextField	*advicePathText;
	NSTextField	*adviceTotalText;
	NSProgressIndicator	*adviceIndicator;
	BOOL		adviceOperation;
	double		adviceDataSize;
}

//********************
// 初期化等
//********************

+ (void)initialize;
- (void)awakeFromNib;
- (void)saveUserDefaults;
//- (void)dealloc;

//********************
// 録音ファイル関係
//********************

- (BOOL)makeSecureRecordSpace;
- (double)moveBackupDataWithPath:(NSString *)folderPath;
- (void)adviceAlertWithPath:(NSString *)folderPath total:(double)totalSize;
- (double)totalDataSize;
- (double)folderSizeWithPath:(NSString *)inDirPath;
//- (void)adviceAlertHandler:(NSTimer *)timer;
- (BOOL)secureRecordSpace;	//必要な録音ファイル領域を確保する
- (double)recordTotalTime;	//現在録音されているファイルの総時間
- (BOOL)deleteOlderFile;	//録音日時の古いファイルを１つ削除する
- (double)needFreeSpace;	//現在の設定で1つの音声データ（波形図を含む）を録音するのに必要な録音域
- (double)maxRecordOne;
- (double)freeSystemSpace;

+ (BOOL)isSoundFile:(NSString *)fileName;	//HiSoundDutyのファイルかどうかをチェックする
+ (BOOL)isImageFile:(NSString *)fileName;
- (BOOL)isLockedFile:(NSString *)fileName;
- (void)lockFile:(NSString *)fileName;
- (void)unlockFile:(NSString *)fileName;
- (BOOL)isHidedFile:(NSString *)fileName;

//********************
// アクセッサ等
//********************

- (NSString *)extensionName;
- (NSString *)temporaryFilePath;
- (NSString *)recordFolder;
- (void)setDirPath:(NSString *)inPath;
- (double)maxRecordTime;
- (void)setMaxRecordTime:(double)inTime;
- (double)maxTotalTime;
- (void)setMaxTotalTime:(double)inTime;
- (AudioStreamBasicDescription)audioFormatInput;
- (SInt64)numDatas;
- (BOOL)adviceOperation;
- (void)setAdviceOperation:(BOOL)inBool;
- (double)adviceDataSize;
- (void)setAdviceDataSize:(double)inSize;
	//-----------------------------
- (void)deleteDotFileInFolder:(NSString *)folderPath;
	//-----------------------------

//********************
// 中継ファイル関係
//********************

- (NSFileHandle *)openTempFile;
- (NSFileHandle *)createRelayFile;
- (NSFileHandle *)openRelayFile;
- (void)deleteRelayFile;

//********************
// オーディオファイル関係
//********************

- (AudioFileID)openAIFFFileWithPath:(NSString *)path;
- (AudioFileID)createAudioFileWithFormat:(AudioStreamBasicDescription)inFormat path:(NSString *)path;
- (SInt64)audioDataByteCount:(AudioFileID)audioFileID;
- (void)setAIFFFileHeader:(NSString *)path channels:(int)channels;

//- (void)actCheckKeeper:(id)sender;

@end
