/* HiPlayer */

#import <Cocoa/Cocoa.h>
#import "MTCoreAudio.h"
#import <AudioToolbox/AudioToolbox.h>

enum {
	Drawer_Play	= 0,
	Drawer_Work	= 1
};

#define SECONDS_OFDAY		(24.0 * 3600.0)
#define HEIGHT_WORKON	249.0
#define HEIGHT_WORKOFF	201.0
#define COLUMN_LOCK		0		//ファイル保護の列番号
#define COLUMN_NOTESTR	6		//備考欄の列番号
#define FILEREAD_INTERVAL	1.0	//音声データを読み込む間隔
#define FILEREAD_SECOND		FILEREAD_INTERVAL*2.0			//一回の読み込みで読み込む音声データの大きさ
#define DISPPOSI_INTERVAL	0.2	//再生位置表示用タイマーの時間間隔
#define COPYBUFFER_SIZE		8192;	//範囲指定コピー時のコピー用バッファ
#define SoundDataSizeRatio	(sizeof(Float32) / sizeof(SInt16))

#define DEF_ADDDATENAME		NO
#define DEF_ADDNOTENAME		NO
#define DEF_STEPFAST		10
#define DEF_VERIFYDEL		YES
#define DEF_ENDLESS			YES

extern NSString *KEY_ADDDATENAME;
extern NSString *KEY_ADDNOTENAME;
extern NSString *KEY_STEPFAST;
extern NSString *KEY_VERIFYDEL;
extern NSString *KEY_ENDLESS;
extern NSString *KEY_CONTROLSTATE;
extern NSString *KEY_WORKSTATE;

@class HiPlayList;
@class HiControler;
@class HiQueBuffer;
@class HiWaveView;

@interface HiPlayer : NSObject
{
	//作業所関係
	NSButton	*lockButton;
	NSButton	*noteButton;
	NSButton	*deleteButton;
	NSButton	*copyButton;
	NSButton	*addDateCheck;
	NSButton	*addNoteCheck;
	NSView		*optionView;
	
	BOOL		verifyDelete;	//削除時に確認するかどうかのフラグ
	
	
	//検証所関係
	HiControler	*controler;
	HiPlayList	*dataSource;
	NSDrawer	*controlDrawer;
	NSButton	*workButton;
	NSButton	*controlButton;
	NSButton	*playButton;
	NSButton	*headButton;
	NSButton	*tailButton;
	NSButton	*backButton;
	NSButton	*fastButton;
	NSSlider	*pointSlider;
	NSTableView *dataView;	//録音ファイル情報
	NSTextField	*totalTimeText;
	NSTextField	*totalSizeText;
	NSTextField *totalFilesText;
	NSTextField	*headTimeText;
	NSTextField	*tailTimeText;
	NSTextField	*currentTimeText;
	NSTextField *quarterTimeText;
	NSTextField *halfTimeText;
	NSTextField *threeQuarterTimeText;
	NSTextField	*stepFastText;
	NSStepper	*stepFastStepper;
	HiWaveView	*waveView;	//波形図表示用
	NSButton	*endlessPlayCheck;
	NSImageView	*playDeviceView;
	
	//範囲選択関係
	NSTextField	*selectHeadText;
	NSTextField	*selectTailText;
	NSTextField	*selectTimeText;
	
	//変数
	int			stepFast;			//早送り時の一回の量
	
	//再生関係
	AudioFileID	playbackFile;		//録音ファイル
	Float32		sampleRate;			//再生データのサンプリング周波数
	int			channels;			//再生データのチャネル数
	SInt64		readPos;			//再生位置
	SInt64		numDatas;			//再生データの大きさ
	BOOL		playbacking;		//再生中フラグ
	BOOL		endlessPlay;		//連続再生するかどうかのフラグ
	int			fastSize;			//早送り、巻戻しの量
	long		lastPoint;			//再生位置スライダーの前回の位置
	SInt16		*readFileBuffer;	//音声ファイル読み込み用バッファ
	Float32		*transformBuffer;	//音声データ変換用バッファ
	HiQueBuffer	*playbackQue;		//音声ファイル読み込み用キューバッファ
	BOOL		endOfSoundFile;		//音声ファイル読み込み終了
	BOOL		stopReadFile;		//一時読み込み停止フラグ
	SInt64		fileReadPos;		//音声ファイルの読み込み位置
	NSTimer		*readTimer;			//音声ファイル読み込み用タイマー
	NSTimer		*dispTimer;			//再生場所表示用タイマー
	
	//ログ関係
	NSString	*deleteFile;
	NSString	*deleteImage;
	NSString	*originalFile;
	NSString	*destinationFile;
	
	BOOL fast;	//デバッグ用
}

//********************
// 初期化等
//********************

- (void)awakeFromNib;
- (void)saveUserDefaults;
//- (void)dealloc;
- (void)initDataSource;

//********************
// 作業所関係
//********************

//ユーティリティ
- (void)setWorkerButtonEnabled:(BOOL)inSwitch;
- (void)_setWorkerButtonEnabled:(BOOL)inSwitch;
- (void)onlyCopyButtonSetEnable;
- (void)editNoteEndHandler:(NSNotification *)aNotification;
- (void)writeNoteWithPath:(NSString *)path note:(NSString *)note;

//アクション
- (void)actLock:(id)sender;
- (void)actNote:(id)sender;
- (void)doubleAction:(id)sender;	//備考欄の編集
- (void)actPlayback:(id)sender;
- (void)actDelete:(id)sender;
- (void)actCopy:(id)sender;
- (void)actWorkDrawerButton:(id)sender;

//アクセッサ
- (BOOL)verifyDelete;
- (void)setVerifyDelete:(BOOL)inBool;
- (HiPlayList *)dataSource;

- (NSString *)deleteFile;
- (NSString *)deleteImage;
- (NSString *)originalFile;
- (NSString *)destinationFile;

//********************
// ユーティリティ
//********************

- (void)changeSelectRow;
- (void)setAllButtonEnabled:(BOOL)inSwitch;
- (void)setPlayerButtonEnabled:(BOOL)inSwitch;
- (void)openDrawer;
- (void)openControlDrawer;
- (void)closeDrawer;
- (void)mainTabSelected;
- (void)mainTabUnselected;
- (void)renewListAndControl;
- (void)dispRecordTime;
- (void)startPlayback:(NSString *)fname;
- (void)stopPlayback;
- (void)openFile:(NSString *)fname;
- (void)pausePlayback:(BOOL)inSwitch;
- (void)imageFileNotificationHandler:(NSNotification *)note;
- (long)adjustReadPosition:(long)inPosition;
- (void)resetPlaybackPoint:(BOOL)inPause;
- (void)drawPositionLine;
- (void)playSourceChanged;
- (void)resetPlaySourceImage;
- (int)transformSoundData:(SInt16 *)source to:(Float32 *)dest size:(int)size ratio:(double)ratio channels:(int)chan;

//範囲選択関係
- (void)selectRangeStartNotificationHandler:(NSNotification *)note;
- (void)selectRangeDragNotificationHandler:(NSNotification *)note;
- (void)selectRangeEndNotificationHandler:(NSNotification *)note;
- (void)selectRangeReleaseNotificationHandler:(NSNotification *)note;
- (void)clearSelectRangeText;
- (void)dispSelectedRangeText:(NSRange)selectedRange;
- (void)dispRangeHeadText:(NSRange)selectedRange;
- (void)dispRangeTailText:(NSRange)selectedRange;
- (void)dispRangeTimeText:(NSRange)selectedRange;
- (double)secondsAtPoint:(int)inPoint;
- (long)headPosition;
- (long)tailPosition;
- (long)selectedLength;

//********************
// アクション
//********************

- (void)actDataView:(id)sender;
- (void)actPlayButton:(id)sender;
- (void)actHeadButton:(id)sender;
- (void)actTailButton:(id)sender;
- (void)actBackButton:(id)sender;
- (void)actFastButton:(id)sender;
- (void)actPointSlider:(id)sender;
- (void)actControlDrawerButton:(id)sender;
- (void)actStepFastText:(id)sender;
- (void)actStepFastStepper:(id)sender;
- (void)actEndlessPlayCheck:(id)sender;

//********************
// オーディオデバイス関係
//********************

- (void)setPlayIOTargetWithDevice:(MTCoreAudioDevice *)playDevice;
- (void)readSoundFileTimer:(NSTimer *)timer;
- (void)readTimerStart;
- (void)readTimerStop;

//********************
// アクセッサ
//********************

- (NSTableView *)dataView;
- (NSImageView *)waveView;
- (BOOL)endlessPlay;
- (void)setEndlessPlay:(BOOL)inBool;
- (void)resetReadPosition:(long)inPosition;

@end
