/* HiPreferences */

#import <Cocoa/Cocoa.h>

enum {
	Index_PrefWatchTab	= 0,
	Index_PrefRecordTab	= 1,
	Index_PrefWorkTab	= 2,
	Index_PrefOtherTab	= 3
};

#define DEF_LASTTABINDEX	0

extern NSString *KEY_LASTTABINDEX;

@class HiControler;
@class HiReserver;
@class HiKeeper;
@class HiMechanic;

@interface HiPrefer : NSObject
{
	HiControler	*controler;
	
	NSTabView		*prefTabView;
	NSDrawer		*preferDrawer;

	NSPopUpButton	*samplePopup;
	NSPopUpButton	*channelsPopup;
	NSPopUpButton	*recordSourcePopup;
	NSPopUpButton	*playSourcePopup;
	NSPopUpButton	*monitorSourcePopup;
	NSPopUpButton	*modePopup;
	NSPopUpButton	*maxRecordPopup;
	NSButton		*autoStartupCheck;
	NSButton		*autoReserveCheck;
	
	NSButton		*okButton;
	NSButton		*preferButton;
	NSTextField		*delayTimeText;
	NSStepper		*delayTimeStepper;
	NSTextField		*freeSpaceText;
	NSTextField		*freeTimeText;
	NSTextField		*maxTotalText;
	NSStepper		*maxTotalStepper;
	NSTextField		*needTotalText;
	NSTextField		*recordFolderText;
	NSButton		*recordFolderButton;
	NSButton		*recordButton;
	NSButton		*sliderAlwaysCheck;
	NSButton		*verifyCheck;
	NSSlider		*opacitySlider;
	NSButton		*hideSoundFileCheck;
	NSTextField		*preBufferSecText;
	NSStepper		*preBufferSecStepper;
	NSButton		*verifyDeleteReserveCheck;
	NSPopUpButton	*markerOptionPopup;
	NSPopUpButton	*markerSoundPopup;
	NSButton		*markerMonitorCheck;
	NSButton		*dispTotalTimeCheck;
	
	NSButton		*defReserveValidCheck;
	NSTextField		*defReserveNameText;
	NSPopUpButton	*defReserveMonthPopup;
	NSPopUpButton	*defReserveDayPopup;
	NSPopUpButton	*defReserveWeekPopup;
	NSPopUpButton	*defReserveRatePopup;
	NSPopUpButton	*defReserveChannelsPopup;
	NSPopUpButton	*defReserveDevicePopup;
	NSPopUpButton	*defReserveModePopup;
	
	NSTextField		*peekHoldTimeText;
	NSStepper		*peekHoldTimeStepper;
	
	NSButton		*writeLogCheck;
	NSButton		*useHighQualityCheck;
	
	int			rateNumber;		//記録するサンプリング周波数の番号
	int			channels;		//記録するチャンネル数
	int			recordSource;	//入力装置
	int			playSource;		//出力装置
	int			monitorSource;	//モニター出力装置
	int			recordMode;		//録音モード
	double		maxRecordTime;	//最大録音時間
	double		maxTotalTime;	//最大累計録音時間
	NSString	*recordFolder;		//音声ファイルの保存場所
	BOOL		sliderAlways;	//自動録音レベルスライダを常時使えるようにするフラグ
	BOOL		verifyDelete;	//削除時に確認するかどうかのフラグ
	float		opacity;			//メイン画面の透明度
	BOOL		hideSoundFile;		//作成した録音ファイルと波形図ファイルを不可視にする
	int			delayTime;		//自動録音停止時の遅延時間
	int			preBufferSec;	//自動録音時に録音スタート前のデータをさかのぼって録音する時間(秒)
	BOOL		verifyDeleteReserve;	//予約削除時に確認をする
	int			markerOption;	//時刻マーカーオプション
	NSString	*markerSound;	//時刻マーカーの音名
	BOOL		markerMonitor;	//モニター時にマーカー音を確認
	BOOL		dispTotalTime;	//録音時間を累計時間表示にする
	BOOL		defReserveValid;
	NSString	*defReserveName;
	int			defReserveMonth;
	int			defReserveDay;
	int			defReserveWeek;
	int			defReserveRate;
	int			defReserveChannels;
	int			defReserveDevice;
	int			defReserveMode;
	double		peekHoldTime;	//瞬間最大値表示時間
	BOOL		writeLog;		//ログを付けるかどうかのフラグ
	BOOL		useHighQuality;
	BOOL		autoStartup;	//
	BOOL		reserveAutoStartup;

	//保存機能関係コントロール
	HiKeeper		*keeper;
	NSButton		*keepOperationCheck;
	NSTextField		*keepFolderText;
	NSButton		*keepFolderButton;
	NSTextField		*keepPeriodText;
	NSStepper		*keepPeriodStepper;
	NSTextField		*commandPathText;
	NSTextField		*argumentStrText;
	NSTextField		*compressRatioText;
	NSStepper		*compressRatioStepper;
	NSButton		*monitorCommandCheck;
	NSButton		*checkKeepActionButton;
	NSTextField		*maxKeepPeriodText;
	
	//保存機能関係変数
	BOOL		keepOperation;	//
	NSString	*keepFolder;	//保存場所
	int			keepPeriod;	//保存期間（単位は日）
	NSString	*commandPath;	//圧縮コマンド
	NSString	*argumentStr;	//圧縮コマンド細部指定文字列
	float		compressRatio;	//圧縮率
	BOOL		monitorCommand;	//圧縮コマンドのモニタースイッチ
	
//----------------------------------------
	HiMechanic		*mechanic;
	NSButton		*adviceOperationCheck;
	NSTextField		*adviceDataSizeText;
	BOOL			adviceOperation;
	double			adviceDataSize;
//----------------------------------------
}

//********************
// 初期化等
//********************

- (void)awakeFromNib;
//- (void)saveUserDefaults;
//- (void)dealloc;

//********************
// ユーティリティ
//********************

- (void)execPrefer;
- (void)exitPrefer;
- (void)resetControlState;
- (void)dispDiskInfo;
- (double)needRecordSpacePerSecond;
- (double)needTotalSpace;
- (double)needRecordSpace;
- (NSArray *)nameArray:(NSArray *)inArray;
- (void)setControlEnabled:(BOOL)inEnabled;
- (void)setPlaySourceEnabled:(BOOL)inEnabled;
- (void)setMarkerControlEnabled:(BOOL)inEnabled;
- (void)renewSampleRatePopup:(NSPopUpButton *)popup source:(int)source select:(int)select;
- (int)sampleRateWithSource:(int)source rateNumber:(int)number;
- (void)autoStartupEnabled;

//保存機能関係ユーティリティ
- (void)compressRatioHandle:(NSNotification *)note;
- (double)maxKeepTimeWithReserveSpace:(double)inReserveSpace
						needPerSecond:(double)inNeedPerSecond
						compressRatio:(float)inCompressRatio;	//現在の設定で保存可能な時間
- (double)reserveSpace;		//録音予約域
- (double)needPerSecond;	//1秒につき必要な録音域
- (BOOL)testFileExists;

//********************
// アクション
//********************

- (void)actExecPrefer:(id)sender;
- (void)actChangePrefer:(id)sender;
- (void)actCanceledPrefer:(id)sender;
- (void)actUsual:(id)sender;
- (void)actMaxRecordTime:(id)sender;
- (void)actMaxTotalText:(id)sender;
- (void)actMaxTotalStepper:(id)sender;
- (void)actDelayTimeText:(id)sender;
- (void)actDelayTimeStepper:(id)sender;
- (void)actDirPath:(id)sender;
- (void)actOpacitySlider:(id)sender;
- (void)actPreBufferSecStepper:(id)sender;
- (void)actPreBufferSecText:(id)sender;
- (void)actMarkerSound:(id)sender;
- (void)actPeekHoldText:(id)sender;
- (void)actPeekHoldStepper:(id)sender;
- (void)actAutoStartupCheck:(id)sender;

//保存機能関係アクション
- (void)actKeepFolderButton:(id)sender;	//未実装
- (void)actKeepPeriodText:(id)sender;
- (void)actKeepPeriodStepper:(id)sender;
- (void)actCompressRatioText:(id)sender;
- (void)actCompressRatioStepper:(id)sender;
- (void)actCheckKeepAction:(id)sender;

//********************
// アクセッサ
//********************

- (NSButton *)preferButton;

@end
