/* HiRecorder */

#import <Cocoa/Cocoa.h>
#import "MTCoreAudio.h"
#import <AudioToolbox/AudioToolbox.h>

enum {
	Condition_Stop		= 0,
	Condition_Record	= 1,
	Condition_Standby	= 2,
	Condition_Timer		= 3
};

enum {
	Mode_Always		= 0,
	Mode_Auto		= 1,
	Mode_Standby	= 2
};

enum {
	Channel_Mono	= 1,
	Channel_Stereo	= 2
};

enum {
	Rate_Poor		= 0,
	Rate_Middle		= 1,
	Rate_High		= 2
};

enum {
	Stop_OFF		= 0,
	Stop_ON			= 1,
	Stop_ReON		= 2
};

enum {
	Thread_Nothing	= 0,
	Thread_StopRecord	= 1,
	Thread_StartRecord	= 2,
	Thread_CreateImage	= 3
};

enum {
	Timer_OFF		= 0,
	Timer_ON		= 1
};

enum {
	Marker_1Minute	= 0,
	Marker_3Minute	= 1,
	Marker_5Minute	= 2,
	Marker_15Minute	= 3,
	Marker_30Minute	= 4
};

enum {
	Method_Manual	= 0,
	Method_Restart	= 1,
	Method_Timer	= 2,
	Method_Halfway	= 3,
	Method_Emergency	= 4,
	Method_Sleep	= 5
};

#define COUNTER_INTERVAL	0.2
#define BLINK_PERIOD		10			//「録音中」文字の点滅間隔
#define BLINK_ONTIME		8			//「録音中」文字の点灯時間
//#define RECORDTIMER_INTERVAL	0.25	//予約タイマーの作動周期
#define RECORDTIMER_INTERVAL	1.0		//予約タイマーの作動周期
#define SYSTEMSOUND_PATH	@"/System/Library/Sounds"
#define WRITEFILE_SLEEPONE	1.0			//ファイル書き込み待機周期単位
//#define WRITEFILE_SLEEPTIME	6			//ファイル書き込み待機周期回数
#define WRITEFILE_SLEEPTIME	5			//ファイル書き込み待機周期回数
//#define MAX_PREBUFFER		(WRITEFILE_SLEEPONE*WRITEFILE_SLEEPTIME*15*44100*2*4)	//録音用キューバッファの大きさ
#define CAUTION_STARTREMAIN	300		//注意を要する予約録音開始前の時間

#define DEF_PREBUFSEC		3
#define DEF_RECORDMODE		Mode_Always
#define DEF_AUTOLEVEL		0
#define DEF_MONITORSWITCH	FALSE
#define DEF_RECORDSWITCH	FALSE
#define DEF_RESERVESWITCH	FALSE
#define DEF_RATENUMBER		Rate_Middle
#define DEF_CHANNELS		Channel_Mono
#define DEF_DELAYTIME		5.0
#define DEF_STARTONHOUR		YES
#define DEF_SLIDERALWAYS	NO
#define DEF_HIDESOUNDFILE	NO
#define DEF_MARKERSWITCH	NO
#define DEF_MARKEROPTION	Marker_1Minute
#define DEF_MARKERSOUND		@""
#define DEF_MARKERMONITOR	NO
#define DEF_WRITEDFRAMES	0
#define DEF_WRITESTART		@""
#define DEF_WRITELAST		@""
#define DEF_DISPTOTALTIME	NO
#define DEF_PROGRESSFILE	@""
#define DEF_AUTOSTARTUP		NO

extern NSString *KEY_RECORDMODE;
extern NSString *KEY_AUTOLEVEL;
extern NSString *KEY_MONITORSWITCH;
extern NSString *KEY_RECORDSWITCH;
extern NSString *KEY_RESERVESWITCH;
extern NSString *KEY_RATENUMBER;
extern NSString *KEY_CHANNELS;
extern NSString *KEY_DELAYTIME;
extern NSString *KEY_STARTONHOUR;
extern NSString *KEY_SLIDERALWAYS;
extern NSString *KEY_HIDESOUNDFILE;
extern NSString *KEY_PREBUFSEC;
extern NSString *KEY_MARKERSWITCH;
extern NSString *KEY_MARKEROPTION;
extern NSString *KEY_MARKERSOUND;
extern NSString *KEY_MARKERMONITOR;
extern NSString *KEY_WRITESTART;
extern NSString *KEY_DISPTOTALTIME;
extern NSString *KEY_PROGRESSFILE;
extern NSString *KEY_AUTOSTARTUP;

@class MTCoreAudioDevice, MTConversionBuffer;
@class HiLevelMeter;
@class HiControler;
@class HiQueBuffer;
@class HiReserverElement;
@class HiMechanic;

@interface HiRecorder : NSObject
{
	//インターフェースビルダー
	HiControler		*controler;
	HiMechanic		*mechanic;
	
	NSButton		*recordButton;
	NSButton		*monitorButton;
	NSSlider		*levelSlider;
	NSTextField		*modeText;
	NSTextField		*conditionText;
	NSTextField		*channelsText;
	NSTextField		*samplingText;
	NSTextField		*recordTimeText;		//録音時間の表示用
	NSTextField		*recordTimeInfoText;
	NSButton		*preferButton;
	NSButton		*reserveButton;			//予約録音ボタン
	NSTextField		*reserveNameText;	//予約名表示用
	NSTextField		*timeInfoText;		//予約録音時刻種別表示用
	NSTextField		*reserveTimeText;	//予約録音開始・終了時刻表示用
	NSButton		*hourButton;	//正時機能ＯＮ／ＯＦＦボタン
	NSButton		*markerButton;		//時刻マーカON/OFFボタン
	NSProgressIndicator *writeIndicator;
	
	//操作盤関係
	BOOL			recordState;	//録音ボタンの状態
	int				recordMode;		//録音モード
	int				condition;		//録音状態
	int				autoLevel;		//自動録音レベル
	NSCalendarDate	*recordStartDate;	//現在の録音開始時間
	NSCalendarDate	*recordStopDate;	//前回の録音終了時間
	double			totalRecordTime;	//録音時間累計
	BOOL			dispTotalTime;	//録音時間を累計時間表示にする
	int				actionMethod;	//録音開始または録音停止の方法
	BOOL			emergency;		//緊急状態（緊急に録音を停止する場合はONにする）
	
	//状態表示点滅用
	int			blinkCounter;		//「録音中」の点滅表示用カウンタ
	NSString	*recordingString;	//「録音中」の点滅表示用文字列
	
	//オーディオデバイス関係
	int					rateNumber;			//記録するサンプリング周波数番号
	int					channels;			//記録するチャンネル数
	int					inputLevel;			//入力レベル
	HiQueBuffer			*soundQue;			//録音用キューバッファ
	SInt16				*transformBuffer;	//データ変換用バッファ
	BOOL				writeThreadError;	//書き込みスレッドエラーフラグ
	BOOL				writeThread;		//データ書き込みスレッド実行中フラグ
	
	//録音関係
	AudioFileID	writeFileID;	//録音ファイル
	SInt64		writePos;		//書き込み位置
	int			preBufferSec;	//自動録音時に録音スタート前のデータをさかのぼって録音する時間(秒)
	int			delayTime;		//自動録音停止時のディレイ時間
	NSDate		*delayCount;	//遅延開始デート
	NSTimer		*dispTimer;		//録音時間表示用タイマー
	BOOL		onTheMinute;		//正分フラグ
//	NSTimer		*minuteTimer;	//１分毎のタイマー
//	NSTimer		*hourTimer;		//１時間毎のタイマー
	BOOL		startOnTheHour;	//毎正時に録音を開始
	BOOL		restarted;		//連続録音時に一旦録音を停止し、再度録音開始した
	BOOL		sliderAlways;	//自動録音レベルスライダを常時使えるようにするフラグ
	BOOL		hideSoundFile;	//作成した録音ファイルと波形図ファイルを不可視にする
	BOOL		firstRecord;	//内部使用、最初の録音フラグ
	BOOL		fileWriteing;	//スレッドでファイル書き込み中
	double		inputCount;		//入力フレーム数
	double		lastInput;
	double		writeCount;		//出力(書き込み)フレーム数
	double		lastWrite;		//最後に書き込んだデータ数（ログで使用）
	long		overflowCount;	//オーバーフローカウンター
	int			overRecordBytes;	//
	double		secondCount;	//録音時間秒数
	NSCalendarDate	*lastCrashDate;	//前回録音中異常終了時の最後の時間
	BOOL		autoStartup;		//起動時自動録音開始
//	BOOL		sleepAtRecording;	//録音中にスリープ状態になった
	NSCalendarDate	*sleepinDate;	//録音中にスリープ状態になった時間
	
	//予約録音関係
	NSTimer		*recordTimer;	//録音予約用タイマー
	double		recordPeriod;	//内部使用、現在作動中のタイマーが終了するまでの時間（秒数）
	BOOL		modeTimer;		//Mode_Timer
	NSDate		*timerStartDate;	//タイマー録音開始日時
	
	//時刻マーカー関係
	BOOL		markerSwitch;	//時刻マーカーを書き込むかどうかのスイッチ
	int			markerOption;	//時刻マーカーオプション（マーカー書き込み時間番号）
	NSString	*markerSound;	//時刻マーカーの音名
	BOOL		markerMonitor;	//モニター時にマーカー音を確認
	int			writeMarker;	//内部使用、時刻マーカー書き込みフラグ兼マーカー音データ残り
	SInt16		*markerBuffer;	//内部使用、マーカー音データ
	int			markerSize;		//内部使用、マーカー音データのサイズ
	
//	BOOL		stopping;		//停止（あるいは録音ファイル交換）処理中
}

//********************
// 初期化等
//********************

- (void)awakeFromNib;
- (void)saveUserDefaults;
//- (void)dealloc;
- (void)initControlState;
- (void)reviveRecordCondition;
- (void)autoRecordStart;
- (void)autoReserveStart;

//********************
// オーディオデバイス関係
//********************

- (void)setRecordIOTargetWithDevice:(MTCoreAudioDevice *)device;
- (void)setMonitorIOTargetWithDevice:(MTCoreAudioDevice *)device;

- (UInt32)writeSoundFile:(UInt32)inSize;

- (void)resetRecordButtonImage;
- (void)resetRecordButtonMarker;
- (void)resetMonitorButtonImage;
- (void)resetMonitorButtonMarker;
- (int)markerIntervalWithNumber;
- (void)dispRecordMode;
- (void)recordSourceChanged;
- (void)monitorSourceChanged;
- (void)protectHowling;

//********************
// スレッド
//********************

- (void)writeSoundFileThread:(id)inArgument;

//********************
// アクセッサ
//********************

- (int)inputLevel;
- (void)controlAutoDelay;
- (BOOL)recordState;
- (int)rateNumber;
- (void)setRateNumber:(int)inRate;
- (double)sampleRate;
- (double)sampleRate:(int)inRateNumber;
- (int)channels;
- (NSString *)channels:(int)inChannels;
- (void)setChannels:(int)inChannels;
- (int)recordMode;
- (void)setRecordMode:(int)inMode;
- (int)delayTime;
- (void)setDelayTime:(int)inTime;
- (BOOL)startOnTheHour;
- (void)setStartOnTheHour:(BOOL)inFlag;
- (BOOL)sliderAlways;
- (void)setSliderAlways:(BOOL)inBool;
- (BOOL)hideSoundFile;
- (void)setHideSoundFile:(BOOL)inBool;
- (int)preBufferSec;
- (void)setPreBufferSec:(int)inSec;
- (NSButton *)recordButton;
- (BOOL)markerSwitch;
- (int)markerOption;
- (void)setMarkerOption:(int)inOption;
- (NSString *)markerSound;
- (void)setMarkerSound:(NSString *)inSound;
- (BOOL)markerMonitor;
- (void)setMarkerMonitor:(BOOL)inBool;
- (BOOL)fileWriteing;
- (id)reserveButton;
- (BOOL)dispTotalTime;
- (void)setDispTotalTime:(BOOL)inBool;
- (NSCalendarDate *)recordStartDate;
- (NSCalendarDate *)recordStopDate;
- (int)condition;

- (double)dataSize;
- (double)lastWrite;
- (NSString *)reserveName;
- (double)totalRecordTime;
- (NSCalendarDate *)lastCrashDate;
- (BOOL)autoStartup;
- (void)setAutoStartup:(BOOL)inBool;

- (BOOL)writeThread;

//- (BOOL)stopping;

//********************
// アクション
//********************

- (void)actRecordButton:(id)sender;
- (void)actMonitorButton:(id)sender;
- (void)actLevelSlider:(id)sender;
- (void)actReserveButton:(id)sender;
- (void)actHourButton:(id)sender;
- (void)_actHourButton;
- (void)actMarkerButton:(id)sender;
//- (void)_actMarkerButton;

//********************
// ユーティリティ
//********************

- (void)mainTabSelected;
- (void)mainTabUnselected;
- (void)makeProgressSoundFile;
- (void)renewalProgressSoundFile;
- (void)deleteProgressSoundFile;
- (void)addOverRecordData;
- (void)delOverRecordData;
- (void)setControlState;
- (void)dispCondition;
- (void)resetCondition;
- (void)startRecording;
- (void)_startRecording;
- (void)stopRecording;
- (void)_stopRecording;
- (void)countRecordTime:(NSTimer *)inTimer;	//表示用タイマー
- (void)onTheMinuteTime;	//マーカー音の挿入用タイマー
- (void)recordTimerPeriodic:(NSTimer *)inTimer;	//予約録音用タイマー
- (void)startReserveRecordWithElement:(HiReserverElement *)element;
- (void)restartRecording;	//連続録音時の再録音開始
- (BOOL)autoDelayState;
- (void)autoDelayStart;
- (void)autoDelayStop;
- (void)recordON;
- (void)recordOFF;
- (void)initMarkerBuffer;
- (void)cancelSoundMarker;
- (void)rescueTempFile;
- (void)writeInformationWithPath:(NSString *)inPath
					   startDate:(NSCalendarDate *)startDate
					    stopDate:(NSCalendarDate *)stopDate
						  option:(NSString *)option;
- (int)actionMethod;
- (void)renewDispSampleRate;
- (void)initSoundQue;
- (UInt32)preBufferSize;
- (int)transformSoundDataSInt16:(SInt16 *)source toSInt16:(SInt16 *)dest size:(int)size ratio:(double)ratio channels:(int)chan;
- (SInt16 *)newTempBuffer:(int)size;
- (SInt16 *)newMarkerBuffer:(int)size;
- (void)emerRecordStop;
- (NSString *)whichHeaderString;	

- (void)willSleepHandle:(NSNotification *)note;	//システムがスリープ状態に入る
- (void)didWakeHandle:(NSNotification *)note;	//システムがスリープから復帰した

//******************************
// ファイル保存(HiKeeper)に必要なもの
//******************************

- (NSString *)headerString;	//データファイルのヘッダ名
- (double)maxRecordTime;	//データファイルの最大記録時間（単位は秒）
- (NSString *)fileExtension;	//データファイルの拡張子
- (double)needPerSecond;	//1秒につき必要な録音域

@end
