/* HiReserver */

#import <Cocoa/Cocoa.h>

#define TIMER_ERROR	-1
#define TIMERNUMBER_MAX		10		//予約タイマー最大数

#define RESERVE_COLUMN_ENABLE	0
#define RESERVE_COLUMN_NAME		1

#define DEF_TIMERDRAWER		NSOffState
#define DEF_VERIFYDELTIMER	YES

#define DEF_DEFRESERVVALID		YES
#define DEF_DEFRESERVNAME		NSLocalizedString(@"NoNamedTimer",@"NoNamedTimer")	//@"名称未設定"
#define DEF_DEFRESERVEMONTH		0
#define DEF_DEFRESERVEDAY		0
#define DEF_DEFRESERVEWEEK		0
#define DEF_DEFRESERVERATE		DEF_RATENUMBER
#define DEF_DEFRESERVECHANNEL	DEF_CHANNELS
#define DEF_DEFRESERVEDEVICE	DEF_RECORDSOURCE
#define DEF_DEFRESERVEMODE		DEF_RECORDMODE
#define DEF_RESERVEAUTOSTARTUP	NO

extern NSString	*KEY_TIMERLIST;
extern NSString *KEY_TIMERDRAWER;
extern NSString *KEY_VERIFYDELTIMER;

extern NSString *KEY_DEFRESERVVALID;
extern NSString *KEY_DEFRESERVNAME;
extern NSString *KEY_DEFRESERVEMONTH;
extern NSString *KEY_DEFRESERVEDAY;
extern NSString *KEY_DEFRESERVEWEEK;
extern NSString *KEY_DEFRESERVERATE;
extern NSString *KEY_DEFRESERVECHANNEL;
extern NSString *KEY_DEFRESERVEDEVICE;
extern NSString *KEY_DEFRESERVEMODE;
extern NSString *KEY_RESERVEAUTOSTARTUP;

@class HiReserverElement;
@class HiReserveList;
@class HiControler;

//********************
// HiReserverクラス
//********************

@interface HiReserver : NSObject
{
	HiControler		*controler;
	NSDrawer		*reserveDrawer;
	NSButton		*drawerButton;
	NSTableView		*dataView;
	HiReserveList	*dataSource;
	NSTextField		*timerNameText;
	NSButton		*timerEnabledCheck;
	NSPopUpButton	*timerStartHourPopup;
	NSPopUpButton	*timerStartMinutePopup;
	NSPopUpButton	*timerStopHourPopup;
	NSPopUpButton	*timerStopMinutePopup;
	NSPopUpButton	*timerSamplePopup;
	NSPopUpButton	*timerChannelsPopup;
	NSPopUpButton	*timerDevicePopup;
	NSPopUpButton	*timerModePopup;
	NSButton		*timerDeleteButton;
	NSButton		*timerAddButton;
	NSButton		*timerCopyButton;
	NSPopUpButton	*timerMonthPopup;
	NSPopUpButton	*timerDayPopup;
	NSPopUpButton	*timerWeekPopup;

	BOOL			verifyDeleteReserve;	//予約削除時に確認をする
	int				defReserveDevice;		//デフォルトの予約録音装置
	BOOL		reserveAutoStartup;		//起動時自動予約録音開始
}

//*****************
// 初期化
//*****************

- (void)saveUserDefaults;
//- (void)dealloc;

//*****************
// デリゲート
//*****************

//*****************
// ユーティリティ
//*****************

- (void)mainTabSelected;
- (void)mainTabUnselected;
- (void)openDrawer;
- (void)closeDrawer;
- (void)renewListAndControl;
- (void)initDataSource;
- (void)initControlState;
- (void)setEnabledTimerButtons;
- (HiReserverElement *)startElement;	//現時刻から１分以内の開始予約があれば予約を返す
- (HiReserverElement *)nextElement;		//現時刻から最も近い録画開始の予約
- (HiReserverElement *)withinElement;	//現時刻が録画開始から終了までの予約
- (int)nextStartTime;
- (void)nextElement:(HiReserverElement **)ioElement second:(int *)ioSecond;
- (int)validReserves;
- (void)setSourcePopup:(NSPopUpButton *)inButton select:(int)inSelect;
- (void)setSamplePopup:(NSPopUpButton *)inButton select:(int)inSelect;

//*****************
// アクション
//*****************

- (void)actDataView:(id)sender;
- (void)actReserveButton:(id)sender;
- (void)actInsertTimer:(id)sender;
- (void)actDeleteTimer:(id)sender;
- (void)actCopyTimer:(id)sender;
- (void)actUsual:(id)sender;
- (void)doubleAction:(id)sender;
- (void)actTimerDevicePopup:(id)sender;

//*****************
// アクセッサ
//*****************

- (HiReserverElement *)elementAtIndex:(int)index;
- (int)count;
- (int)enables;
- (void)insertElement:(HiReserverElement *)element atIndex:(int)index;
- (void)removeElementAtIndex:(int)index;

- (BOOL)verifyDeleteReserve;
- (void)setVerifyDeleteReserve:(BOOL)inBool;

- (BOOL)defReserveValid;
- (void)setDefReserveValid:(BOOL)inBool;
- (NSString *)defReserveName;
- (void)setDefReserveName:(NSString *)inName;
- (int)defReserveMonth;
- (void)setDefReserveMonth:(int)inMonth;
- (int)defReserveDay;
- (void)setDefReserveDay:(int)inDay;
- (int)defReserveWeek;
- (void)setDefReserveWeek:(int)inWeek;
- (int)defReserveRate;
- (void)setDefReserveRate:(int)inRate;
- (int)defReserveChannels;
- (void)setDefReserveChannels:(int)inChannels;
- (int)defReserveDevice;
- (void)setDefReserveDevice:(int)inDevice;
- (void)changeDefReserveDevice:(int)inDevice;
- (int)defReserveMode;
- (void)setDefReserveMode:(int)inMode;
- (BOOL)reserveAutoStartup;
- (void)setReserveAutoStartup:(BOOL)inBool;

@end

//********************
// HiReserverElementクラス
//********************

@interface HiReserverElement : NSObject
{
	//以下の変数の順番を変更しないこと（特にdevice）
	NSString	*name;
	BOOL		enabled;
	int			start;
	int			stop;	//停止時刻（秒数）
	int			mode;
	int			sample;
	int			channels;
	int			device;	//7
	int			month;
	int			day;
	int			week;
}

- (id)initWithValueArray:(NSArray *)inValue;
- (id)clone;
- (NSArray *)valueArray;
- (BOOL)isEqualElement:(HiReserverElement *)inElement;

- (NSString *)name;
- (void)setName:(NSString *)inName;
- (BOOL)enabled;
- (void)setEnabled:(BOOL)inBool;
- (int)start;
- (int)startHour;
- (int)startMinute;
- (void)setStart:(int)inStart;
- (int)stop;
- (int)stopHour;
- (int)stopMinute;
- (int)stopSecond;
- (void)setStop:(int)inStop;
- (int)mode;
- (void)setMode:(int)inMode;
- (int)sample;
- (void)setSample:(int)inSample;
- (int)channels;
- (void)setChannels:(int)inChannels;
- (int)device;
- (void)setDevice:(int)inDevice;
- (int)month;
- (void)setMonth:(int)inMonth;
- (int)day;
- (void)setDay:(int)inDay;
- (int)week;
- (void)setWeek:(int)inWeek;

@end
